// TODO: should vee-validate be imported here?
// It should be something like a peer dependency
// Vue, VueRouter, Vuex could also be peer dependencies,
// so they wouldn't be passed as arguments
import VeeValidate from 'vee-validate';
import Keycloak from 'keycloak-js';
import fenceStore from '../../store';
import api from './api';
import processApiPaths from './processApiPaths';

class Configurator {
    constructor(args) {
        this.validate(args);

        Object.assign(this, args);
    }

    validate(args) {
        if (!args) {
            this.error = 'Missing required arguments object';

            return;
        }

        const missingArguments = [];
        const requiredArguments = [
            'Vue',
            'App',
            'Vuex',
            'VueRouter',
            'routes',
            'apiBaseUrl',
        ];

        requiredArguments.forEach((argument) => {
            if (!args[argument]) missingArguments.push(argument);
        });

        if (missingArguments.length) {
            this.error = (`Missing required argument(s): [${missingArguments}]`);
        }
    }

    configureVue() {
        const { Vue } = this;

        // TODO: implement dev/production mode
        // Is it necessary here?
        // Should it use process.env.NODE_ENV !== 'production'?
        Vue.config.productionTip = false;
    }

    configureVuex() {
        const {
            Vue,
            Vuex,
            store: systemStore,
        } = this;

        Vue.use(Vuex);

        // TODO: implement dev/production mode
        const debug = true; // process.env.NODE_ENV !== 'production';

        const store = new Vuex.Store({
            strict: debug,
            ...systemStore,
        });

        store.registerModule('fence', fenceStore);

        const updateTitleSubscription = () => {
            const { title } = store.state.fence.document;

            document.title = title;
        };

        const subscriptionMapper = {
            'fence/document/updateTitle': updateTitleSubscription,
        };

        store.subscribe((mutation) => {
            const subscription = subscriptionMapper[mutation.type] || null;

            if (subscription) subscription();
        });

        this.vuex = store;
    }

    configureVueRouter() {
        const {
            Vue,
            VueRouter,
            routes,
            vuex: store,
        } = this;

        Vue.use(VueRouter);

        const router = new VueRouter({ routes });

        // Adapted from https://alligator.io/vuejs/vue-router-modify-head/
        // This callback runs before every route change, including on page load.
        router.beforeEach((to, from, next) => {
            // This goes through the matched routes from last to first, finding the closest
            // route with a title. Eg. if we have /some/deep/nested/route and /some, /deep,
            // and /nested have titles, nested's will be chosen.
            const nearestWithTitle = [...to.matched]
                .reverse()
                .find(route => route.meta && route.meta.documentTitleSuffix);

            // If a route with a title suffix was found, set
            // the document (page) title to that value.
            if (nearestWithTitle) {
                store.commit(
                    'fence/document/setTitleSuffix',
                    nearestWithTitle.meta.documentTitleSuffix,
                );
                store.commit('fence/document/updateTitle');
            }

            next();
        });

        this.router = router;
    }

    configureApi() {
        const { Vue, vuex, apiBaseUrl } = this;
        const { axios } = api;

        api.store = vuex;
        axios.defaults.baseURL = apiBaseUrl;

        this.configureApiMock();
        this.configureApiPaths();

        Vue.prototype.$api = axios;
    }

    configureApiMock() {
        const { Vue, apiMockDelay = 500 } = this;
        const { axios } = api;

        Vue.prototype.$apiMock = () => {
            const mock = Vue.prototype.$apiMockAdapter;

            if (mock) return mock;

            // Disables linter since this is the way MockAdapter should
            // be imported. $apiMock can never be used in a production
            // environment, it will break (as it should).
            // eslint-disable-next-line global-require, import/no-extraneous-dependencies
            const MockAdapter = require('axios-mock-adapter');
            const mockAdapter = new MockAdapter(axios, { delayResponse: apiMockDelay });

            Vue.prototype.$apiMockAdapter = mockAdapter;

            return mockAdapter;
        };
    }

    configureApiPaths() {
        const { Vue, apiPaths } = this;

        if (!apiPaths) return;

        Vue.prototype.$apiPaths = processApiPaths(apiPaths);
    }

    configureVeeValidate() {
        const { Vue } = this;

        // TODO: It would be better to install this only in
        // the FenceInput component, but I wasn't able to
        Vue.use(VeeValidate, {
            inject: false,
            classes: true,
            classNames: {
                valid: 'is-valid',
                invalid: 'is-invalid',
            },
        });
    }

    configure() {
        if (this.error) {
            console.error(this.error);

            return;
        }

        this.configureVue();
        this.configureVuex();
        this.configureVueRouter();
        this.configureApi();
        this.configureVeeValidate();

        const authOptions = this.auth || {};
        if (authOptions.sso === 'keycloak') {
            this.configureKeycloak(authOptions.clientId);
        } else {
            this.createVueInstance();
        }
    }

    configureKeycloak(clientId) {
        const initOptions = {
            url: 'https://auth.cern.ch/auth',
            realm: 'cern',
            clientId,
            onLoad: 'login-required',
        };

        const keycloak = Keycloak(initOptions);

        keycloak.init({
            onLoad: initOptions.onLoad,
            flow: 'implicit',
        }).success((auth) => {
            if (!auth) {
                window.location.reload();
            }

            this.createVueInstance();

            sessionStorage.setItem('accessToken', keycloak.token);

            setTimeout(() => {
                keycloak.updateToken(60);
            }, 60000);
        }).error((error) => {
            console.log(error);
        });
    }

    createVueInstance() {
        const {
            Vue,
            App,
            router,
            vuex: store,
        } = this;

        new Vue({
            render: h => h(App),
            router,
            store,
        }).$mount('#app');
    }
}

function main(args) {
    const configurator = new Configurator(args);

    configurator.configure();
}

export default main;
